@@ -1,6 +1,6 @@
 ;;; cmuscheme.el --- Scheme process in a buffer. Adapted from tea.el.
 
-;; Copyright (C) 1988, 1994 Free Software Foundation, Inc.
+;; Copyright (C) 1988, 1994, 1997 Free Software Foundation, Inc.
 
 ;; Author: Olin Shivers <olin.shivers@cs.cmu.edu>
 ;; Maintainer: FSF
@@ -23,11 +23,11 @@
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
-;;; Synched up with: FSF 19.34.
+;;; Synched up with: FSF 20.2
 
 ;;; Commentary:
 
-;;    This is a customisation of comint-mode (see comint.el)
+;;    This is a customization of comint-mode (see comint.el)
 ;;
 ;; Written by Olin Shivers (olin.shivers@cs.cmu.edu). With bits and pieces
 ;; lifted from scheme.el, shell.el, clisp.el, newclisp.el, cobol.el, et al..
@@ -80,8 +80,8 @@
 (defvar inferior-scheme-mode-map nil)
 
 (cond ((not inferior-scheme-mode-map)
-       (setq inferior-scheme-mode-map
-	     (copy-keymap comint-mode-map))
+       (setq inferior-scheme-mode-map (make-sparse-keymap))
+       (set-keymap-parents inferior-scheme-mode-map comint-mode-map)
        (define-key inferior-scheme-mode-map "\M-\C-x" ;gnu convention
 	           'scheme-send-definition)
        (define-key inferior-scheme-mode-map "\C-x\C-e" 'scheme-send-last-sexp)
@@ -182,10 +182,24 @@ Defaults to a regexp ignoring all inputs of 0, 1, or 2 letters.")
 		 (scheme-args-to-list (substring string pos
 						 (length string)))))))))
 
+(defun scheme-list-to-args (args)
+  (let* ((str (apply #'concat
+		   (mapcar #'(lambda (arg)
+			       (concat " " arg))
+			   args)))
+	 (l (length str)))
+    (if (zerop l)
+	str
+      (substring str 1 (length str)))))
+
 (defvar scheme-program-name "scheme"
   "*Program invoked by the run-scheme command")
 
-(defun run-scheme (cmd)
+(defvar scheme-program-args nil
+  "*Command-line arguments to `scheme-program-name'.")
+
+;;; *don't* ;;;###autoload in XEmacs, since xscheme.el is our default.
+(defun run-scheme (cmd args)
   "Run an inferior Scheme process, input and output via buffer *scheme*.
 If there is a process already running in `*scheme*', switch to that buffer.
 With argument, allows you to edit the command line (default is value
@@ -195,16 +209,22 @@ of `scheme-program-name').  Runs the hooks `inferior-scheme-mode-hook'
 
   (interactive (list (if current-prefix-arg
 			 (read-string "Run Scheme: " scheme-program-name)
-			 scheme-program-name)))
+			 scheme-program-name)
+		      (if current-prefix-arg
+			  (scheme-args-to-list
+			   (read-string "Command-line arguments: " 
+					(scheme-list-to-args scheme-program-args)))
+			scheme-program-args)))
   (if (not (comint-check-proc "*scheme*"))
-      (let ((cmdlist (scheme-args-to-list cmd)))
-	(set-buffer (apply 'make-comint "scheme" (car cmdlist)
-			   nil (cdr cmdlist)))
+      (progn
+	(set-buffer (apply 'make-comint "scheme" cmd
+			   nil args))
 	(inferior-scheme-mode)))
   (setq scheme-program-name cmd)
+  (setq scheme-program-args args)
   (setq scheme-buffer "*scheme*")
   (pop-to-buffer "*scheme*"))
-;;;;;###autoload (add-hook 'same-window-buffer-names "*scheme*")
+;;; *don't* ;;;###autoload (add-hook 'same-window-buffer-names "*scheme*")
 
 (defun scheme-send-region (start end)
   "Send the current region to the inferior Scheme process."
